==> Sah-Schemas-Str-0.012.tar.gz: Sah-Schemas-Str-0.012/lib/Sah/Schema/percent_str.pm <==
package Sah::Schema::percent_str;

use strict;

use Regexp::Pattern::Float;

our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
our $DATE = '2022-09-22'; # DATE
our $DIST = 'Sah-Schemas-Str'; # DIST
our $VERSION = '0.012'; # VERSION

our $schema = [str => {
    summary => 'A number in percent form',
    match => qr/\A$Regexp::Pattern::Float::RE{float_decimal}{pat}%\z/,

    description => <<'_',

This schema accepts floating number followed by percent sign. Unlike the
`percent` schema from <pm:Sah::Schemas::Float>, The percent sign will not be
removed nor the number be converted to decimal (e.g. 50% to 0.5).

_
    examples => [
        {value=>'', valid=>0, summary=>'No percent sign'},
        {value=>'1', valid=>0, summary=>'No percent sign'},
        {value=>'1%', valid=>1},
        {value=>'1 %', valid=>0, summary=>'Whitespace before percent sign is currently not allowed'},
        {value=>'-1.23%', valid=>1},
        {value=>'%', valid=>0, summary=>'Percent sign only'},
    ],

}];

1;
# ABSTRACT: A number in percent form

__END__

=pod

=encoding UTF-8

=head1 NAME

Sah::Schema::percent_str - A number in percent form

=head1 VERSION

This document describes version 0.012 of Sah::Schema::percent_str (from Perl distribution Sah-Schemas-Str), released on 2022-09-22.

=head1 SYNOPSIS

=head2 Sample data and validation results against this schema

 ""  # INVALID (No percent sign)

 1  # INVALID (No percent sign)

 "1%"  # valid

 "1 %"  # INVALID (Whitespace before percent sign is currently not allowed)

 "-1.23%"  # valid

 "%"  # INVALID (Percent sign only)

=head2 Using with Data::Sah

To check data against this schema (requires L<Data::Sah>):

 use Data::Sah qw(gen_validator);
 my $validator = gen_validator("percent_str*");
 say $validator->($data) ? "valid" : "INVALID!";

The above schema returns a boolean result (true if data is valid, false if
otherwise). To return an error message string instead (empty string if data is
valid, a non-empty error message otherwise):

 my $validator = gen_validator("percent_str", {return_type=>'str_errmsg'});
 my $errmsg = $validator->($data);
 
 # a sample valid data
 $data = "1%";
 my $errmsg = $validator->($data); # => ""
 
 # a sample invalid data
 $data = 1;
 my $errmsg = $validator->($data); # => "Must match regex pattern qr(\\A(?^:[+-]?(?:[0-9]+(?:\\.[0-9]*)?|[0-9]*\\.[0-9]+))%\\z)"

Often a schema has coercion rule or default value, so after validation the
validated value is different. To return the validated (set-as-default, coerced,
prefiltered) value:

 my $validator = gen_validator("percent_str", {return_type=>'str_errmsg+val'});
 my $res = $validator->($data); # [$errmsg, $validated_val]
 
 # a sample valid data
 $data = "1%";
 my $res = $validator->($data); # => ["","1%"]
 
 # a sample invalid data
 $data = 1;
 my $res = $validator->($data); # => ["Must match regex pattern qr(\\A(?^:[+-]?(?:[0-9]+(?:\\.[0-9]*)?|[0-9]*\\.[0-9]+))%\\z)",1]

Data::Sah can also create validator that returns a hash of detailed error
message. Data::Sah can even create validator that targets other language, like
JavaScript, from the same schema. Other things Data::Sah can do: show source
code for validator, generate a validator code with debug comments and/or log
statements, generate human text from schema. See its documentation for more
details.

=head2 Using with Params::Sah

To validate function parameters against this schema (requires L<Params::Sah>):

 use Params::Sah qw(gen_validator);

 sub myfunc {
     my @args = @_;
     state $validator = gen_validator("percent_str*");
     $validator->(\@args);
     ...
 }

=head2 Using with Perinci::CmdLine::Lite

To specify schema in L<Rinci> function metadata and use the metadata with
L<Perinci::CmdLine> (L<Perinci::CmdLine::Lite>) to create a CLI:

 # in lib/MyApp.pm
 package
   MyApp;
 our %SPEC;
 $SPEC{myfunc} = {
     v => 1.1,
     summary => 'Routine to do blah ...',
     args => {
         arg1 => {
             summary => 'The blah blah argument',
             schema => ['percent_str*'],
         },
         ...
     },
 };
 sub myfunc {
     my %args = @_;
     ...
 }
 1;

 # in myapp.pl
 package
   main;
 use Perinci::CmdLine::Any;
 Perinci::CmdLine::Any->new(url=>'/MyApp/myfunc')->run;

 # in command-line
 % ./myapp.pl --help
 myapp - Routine to do blah ...
 ...

 % ./myapp.pl --version

 % ./myapp.pl --arg1 ...

=head1 DESCRIPTION

This schema accepts floating number followed by percent sign. Unlike the
C<percent> schema from L<Sah::Schemas::Float>, The percent sign will not be
removed nor the number be converted to decimal (e.g. 50% to 0.5).

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/Sah-Schemas-Str>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-Sah-Schemas-Str>.

=head1 SEE ALSO

L<Sah::Schema::percent>

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 CONTRIBUTING


To contribute, you can send patches by email/via RT, or send pull requests on
GitHub.

Most of the time, you don't need to build the distribution yourself. You can
simply modify the code, then test via:

 % prove -l

If you want to build the distribution (e.g. to try to install it locally on your
system), you can install L<Dist::Zilla>,
L<Dist::Zilla::PluginBundle::Author::PERLANCAR>,
L<Pod::Weaver::PluginBundle::Author::PERLANCAR>, and sometimes one or two other
Dist::Zilla- and/or Pod::Weaver plugins. Any additional steps required beyond
that are considered a bug and can be reported to me.

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2022, 2020 by perlancar <perlancar@cpan.org>.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Sah-Schemas-Str>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=cut

==> Sah-Schemas-Str-0.013.tar.gz: Sah-Schemas-Str-0.013/lib/Sah/Schema/percent_str.pm <==
package Sah::Schema::percent_str;

use strict;

use Regexp::Pattern::Float;

our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
our $DATE = '2022-09-23'; # DATE
our $DIST = 'Sah-Schemas-Str'; # DIST
our $VERSION = '0.013'; # VERSION

our $schema = [str => {
    summary => 'A number in percent form, e.g. "10.5%"',
    match => qr/\A$Regexp::Pattern::Float::RE{float_decimal}{pat}%\z/,

    description => <<'_',

This schema accepts floating number followed by percent sign. Unlike the
`percent` schema from <pm:Sah::Schemas::Float>, The percent sign will not be
removed nor the number be converted to decimal (e.g. 50% to 0.5).

_
    examples => [
        {value=>'', valid=>0, summary=>'No percent sign'},
        {value=>'1', valid=>0, summary=>'No percent sign'},
        {value=>'1%', valid=>1},
        {value=>'1 %', valid=>0, summary=>'Whitespace before percent sign is currently not allowed'},
        {value=>'-1.23%', valid=>1},
        {value=>'%', valid=>0, summary=>'Percent sign only'},
    ],

}];

1;
# ABSTRACT: A number in percent form, e.g. "10.5%"

__END__

=pod

=encoding UTF-8

=head1 NAME

Sah::Schema::percent_str - A number in percent form, e.g. "10.5%"

=head1 VERSION

This document describes version 0.013 of Sah::Schema::percent_str (from Perl distribution Sah-Schemas-Str), released on 2022-09-23.

=head1 SYNOPSIS

=head2 Sample data and validation results against this schema

 ""  # INVALID (No percent sign)

 1  # INVALID (No percent sign)

 "1%"  # valid

 "1 %"  # INVALID (Whitespace before percent sign is currently not allowed)

 "-1.23%"  # valid

 "%"  # INVALID (Percent sign only)

=head2 Using with Data::Sah

To check data against this schema (requires L<Data::Sah>):

 use Data::Sah qw(gen_validator);
 my $validator = gen_validator("percent_str*");
 say $validator->($data) ? "valid" : "INVALID!";

The above schema returns a boolean result (true if data is valid, false if
otherwise). To return an error message string instead (empty string if data is
valid, a non-empty error message otherwise):

 my $validator = gen_validator("percent_str", {return_type=>'str_errmsg'});
 my $errmsg = $validator->($data);
 
 # a sample valid data
 $data = "-1.23%";
 my $errmsg = $validator->($data); # => ""
 
 # a sample invalid data
 $data = "1 %";
 my $errmsg = $validator->($data); # => "Must match regex pattern qr(\\A(?^:[+-]?(?:[0-9]+(?:\\.[0-9]*)?|[0-9]*\\.[0-9]+))%\\z)"

Often a schema has coercion rule or default value, so after validation the
validated value is different. To return the validated (set-as-default, coerced,
prefiltered) value:

 my $validator = gen_validator("percent_str", {return_type=>'str_errmsg+val'});
 my $res = $validator->($data); # [$errmsg, $validated_val]
 
 # a sample valid data
 $data = "-1.23%";
 my $res = $validator->($data); # => ["","-1.23%"]
 
 # a sample invalid data
 $data = "1 %";
 my $res = $validator->($data); # => ["Must match regex pattern qr(\\A(?^:[+-]?(?:[0-9]+(?:\\.[0-9]*)?|[0-9]*\\.[0-9]+))%\\z)","1 %"]

Data::Sah can also create validator that returns a hash of detailed error
message. Data::Sah can even create validator that targets other language, like
JavaScript, from the same schema. Other things Data::Sah can do: show source
code for validator, generate a validator code with debug comments and/or log
statements, generate human text from schema. See its documentation for more
details.

=head2 Using with Params::Sah

To validate function parameters against this schema (requires L<Params::Sah>):

 use Params::Sah qw(gen_validator);

 sub myfunc {
     my @args = @_;
     state $validator = gen_validator("percent_str*");
     $validator->(\@args);
     ...
 }

=head2 Using with Perinci::CmdLine::Lite

To specify schema in L<Rinci> function metadata and use the metadata with
L<Perinci::CmdLine> (L<Perinci::CmdLine::Lite>) to create a CLI:

 # in lib/MyApp.pm
 package
   MyApp;
 our %SPEC;
 $SPEC{myfunc} = {
     v => 1.1,
     summary => 'Routine to do blah ...',
     args => {
         arg1 => {
             summary => 'The blah blah argument',
             schema => ['percent_str*'],
         },
         ...
     },
 };
 sub myfunc {
     my %args = @_;
     ...
 }
 1;

 # in myapp.pl
 package
   main;
 use Perinci::CmdLine::Any;
 Perinci::CmdLine::Any->new(url=>'/MyApp/myfunc')->run;

 # in command-line
 % ./myapp.pl --help
 myapp - Routine to do blah ...
 ...

 % ./myapp.pl --version

 % ./myapp.pl --arg1 ...

=head1 DESCRIPTION

This schema accepts floating number followed by percent sign. Unlike the
C<percent> schema from L<Sah::Schemas::Float>, The percent sign will not be
removed nor the number be converted to decimal (e.g. 50% to 0.5).

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/Sah-Schemas-Str>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-Sah-Schemas-Str>.

=head1 SEE ALSO

L<Sah::Schema::percent>

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 CONTRIBUTING


To contribute, you can send patches by email/via RT, or send pull requests on
GitHub.

Most of the time, you don't need to build the distribution yourself. You can
simply modify the code, then test via:

 % prove -l

If you want to build the distribution (e.g. to try to install it locally on your
system), you can install L<Dist::Zilla>,
L<Dist::Zilla::PluginBundle::Author::PERLANCAR>,
L<Pod::Weaver::PluginBundle::Author::PERLANCAR>, and sometimes one or two other
Dist::Zilla- and/or Pod::Weaver plugins. Any additional steps required beyond
that are considered a bug and can be reported to me.

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2022, 2020 by perlancar <perlancar@cpan.org>.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Sah-Schemas-Str>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=cut
